#include <utility> // Para std::swap

#include "scoreboard.hpp"

// Constroi um placar com até limit maiores valores.
Scoreboard::Scoreboard(unsigned int limit) : _limit(limit) {}

// Insere new_score, se for um dos maiores.
void Scoreboard::insert(double new_score) {
  // Insere no final (possivelmente fora de ordem), caso apropriado.
  if (_scores.size() < _limit) {
    // Se ainda não temos _limit elementos, inserimos o novo.
    _scores.push_back(new_score);
  } else if (new_score > _scores[_limit - 1]) {
    // Se já temos _limit elementos e o novo é maior do que o menor dos
    // guardados, descartamos o menor trocando ele pelo novo.
    _scores[_limit - 1] = new_score;
  }

  // Se houve inserção, precisamos colocar o novo valor (último em _scores) na
  // posição correta. Como os os outros valores já estão ordenados, basta
  // subir o novo valor até a posição correta dele. Se não houve inserção, o
  // primeiro teste _scores[i-1] < _scores[i] já falha, interrompendo o for
  // sem executar nenhum swap.
  for (size_t i = _scores.size() - 1;         // Começamos do final.
       i >= 1 && _scores[i - 1] < _scores[i]; // Terminamos ao achar um maior.
       --i) {
    // Trocamos caso o anterior seja menor.
    std::swap(_scores[i - 1], _scores[i]);
  }
}

// Retorna os maiores scores registrados até o momento da chamada, do
// maior para o menor.
std::vector<double> Scoreboard::largest_scores() {
  // Os valores já estão em _scores na ordem desejada.
  // Retornamos uma cópia.
  return _scores;
}
